class ClickUz_Errors:
    errors = {}
    def __init__(self):
        self.errors = {
            "0" : "Success",
            "-1" : "SIGN CHECK FAILED!",
            "-2" : "Incorrect parameter amount",
            "-3" : "Action not found",
            "-4" : "Already paid",
            "-5" : "User does not exist",
            "-6" : "Transaction does not exist",
            "-7" : "Failed to update user",
            "-8" : "Error in request from click",
            "-9" : "Transaction cancelled"
        }
    def get_error(self, error):
        return self.errors.get(str(error))


from flask import Blueprint, jsonify, request
import hashlib, datetime


def check_in_req(data):
    return 'click_trans_id' in data\
        and 'service_id' in data\
        and 'click_paydoc_id' in data\
        and 'merchant_trans_id' in data\
        and 'amount' in data\
        and 'action' in data\
        and 'error' in data\
        and 'error_note' in data\
        and 'sign_time' in data\
        and 'sign_string' in data


def check_hash_code(data,secret_key):
    s = f"{data['click_trans_id']}{data['service_id']}{secret_key}{data['merchant_trans_id']}{data['amount']}{data['action']}{data['sign_time']}"
    s = hashlib.md5(s.encode('utf-8')).hexdigest()
    return s == data['sign_string']


def check_hash_code_prepare(data, secret_key, t_id):
    s = f"{data['click_trans_id']}{data['service_id']}{secret_key}{data['merchant_trans_id']}{t_id}{data['amount']}{data['action']}{data['sign_time']}"
    s = hashlib.md5(s.encode('utf-8')).hexdigest()
    return s == data['sign_string']


def Create_Blueprint(click):
    bp = Blueprint('click', __name__, url_prefix='/click')
    
    @bp.route('/prepare', methods=['POST'])
    def prepare():
        data = dict(request.form)
        if not check_in_req(data):
            return jsonify({"status" : "error"}), 400    
        model = click.model
        new_transaction = model(
            click_trans_id = data.get('click_trans_id'),
            service_id = data.get('service_id'),
            click_paydoc_id = data.get('click_paydoc_id'),
            merchant_trans_id = data.get('merchant_trans_id'),
            amount = data.get('amount'),
            action = data.get('action'),
            error = data.get('error'),
            error_note = data.get('error_note'),
            sign_time = datetime.datetime.strptime(data.get('sign_time'), '%Y-%m-%d %H:%M:%S'),
            sign_string = data.get('sign_string')
        )
        click.db.session.add(new_transaction)
        click.db.session.commit() 
        if not click.validator(data):
            res = new_transaction.response()
            res['error'] = '-5'
            res['error_note'] = click.errors.get_error('-5')
            click.db.session.delete(new_transaction)
            click.db.session.commit()
            return jsonify(res), 200
        
        if not check_hash_code(data, click.click_secret):
            new_transaction.error = '-1'
            new_transaction.error_note = click.errors.get_error('-1')
            click.db.session.commit()
            res = new_transaction.response()
            click.db.session.delete(new_transaction)
            click.db.session.commit()
            return jsonify(res), 200
        return jsonify(new_transaction.response()), 200
    
    @bp.route('/complete', methods=['POST'])
    def complete():
        data = dict(request.form)
        if not check_in_req(data):
            return jsonify({"status" : "error"}), 400
        model = click.model
        transaction = model.query.filter_by(id = data.get('merchant_prepare_id')).first()
        if not transaction:
            res = {
                    "click_trans_id" : data['click_trans_id'],
                    "merchant_trans_id" : data['merchant_trans_id'],
                    "merchant_confirm_id" : data['merchant_prepare_id'],
                    "error" : data['error'],
                    "error_note" : data['error_note']
                }
            res['error'] = '-6'
            res['error_note'] = click.errors.get_error('-6')
            return jsonify(res), 200
        if data.get('error') == '-5017' or transaction.error == -9:
            click.callback(transaction)
            transaction.error = -9
            transaction.error_note = click.errors.get_error('-9')
            click.db.session.commit()
            res = transaction.response_confirm()
            return jsonify(res), 200
        if not check_hash_code_prepare(data, click.click_secret, transaction.id):
            res = transaction.response_confirm()
            res['error'] = '-1'
            res['error_note'] = click.errors.get_error('-1')
            return jsonify(res), 200
        if transaction.amount != float(data.get('amount')):
            res = transaction.response_confirm()
            res['error'] = '-2'
            res['error_note'] = click.errors.get_error('-2')
            return jsonify(res), 200
        if transaction.action == 1:
            res = transaction.response_confirm()
            res['error'] = '-4'
            res['error_note'] = click.errors.get_error('-4')
            return jsonify(res), 200
            
        transaction.error = 0
        transaction.error_note = click.errors.get_error('0')
        transaction.action = 1
        click.db.session.commit()
        click.callback(transaction)
        return jsonify(transaction.response_confirm()), 200
    return bp





from flask import Flask, Blueprint
from .blueprint import Create_Blueprint
from .exceptions import Flask_ClickUz_Exception
from .errors import ClickUz_Errors
class Click:
    def __init__(self, db):
        self.validator = None
        self.db = db
        self.callback = None
        click_secret = None
        click_service_id = None
        click_merchant_id = None
        self.errors = ClickUz_Errors()
        class Click_Transaction(db.Model):
            __tablename__ = 'click_transaction'
            id = db.Column(db.Integer, primary_key=True)
            click_trans_id = db.Column(db.Integer, unique=True)
            service_id = db.Column(db.Integer)
            click_paydoc_id = db.Column(db.Integer)
            merchant_trans_id = db.Column(db.String(255))
            amount = db.Column(db.Float)
            action = db.Column(db.Integer)
            error = db.Column(db.Integer)
            error_note = db.Column(db.String(255))
            sign_time = db.Column(db.DateTime)
            sign_string = db.Column(db.String(255))
            
            
            def __repr__(self):
                return "%s"%(self.merchant_trans_id)
            
    
            def response(self):
                return {
                    "click_trans_id" : self.click_trans_id,
                    "merchant_trans_id" : self.merchant_trans_id,
                    "merchant_prepare_id" : self.id,
                    "error" : self.error,
                    "error_note" : self.error_note
                }
            def response_confirm(self):
                return {
                    "click_trans_id" : self.click_trans_id,
                    "merchant_trans_id" : self.merchant_trans_id,
                    "merchant_confirm_id" : self.id,
                    "error" : self.error,
                    "error_note" : self.error_note
                }
            
        self.model = Click_Transaction   
        
        pass
    def init_app(self, app: Flask):
        click_secret = app.config.get('CLICK_SECRET')
        click_service_id = app.config.get('CLICK_SERVICE_ID')
        click_merchant_id = app.config.get('CLICK_MERCHANT_ID')
        if not click_secret:
            raise Flask_ClickUz_Exception('CLICK_SECRET is not set')
        if not click_service_id:
            raise Flask_ClickUz_Exception('CLICK_SERVICE_ID is not set')
        if not click_merchant_id:
            raise Flask_ClickUz_Exception('CLICK_MERCHANT_ID is not set')
        if not self.validator:
            raise Flask_ClickUz_Exception('Validator is not set, Set validator with Register_Validator method')
        if not self.callback:
            raise Flask_ClickUz_Exception('Callback is not set, Set callback with Register_Callback method')
        self.click_secret = click_secret
        self.click_service_id = click_service_id
        self.click_merchant_id = click_merchant_id
        bp = Create_Blueprint(self)
        app.register_blueprint(bp)
        
    
    def Register_Validator(self, validator):
        """Calls when click sends request to server

        Args:
            validator (function): Callback function, it takes click data as argument
        """ 
        self.validator = validator
    
    def Register_Callback(self, callback):
        """Calls when transaction is completed or cancelled, 
        if transaction is completed, transaction.action is 1
        else transaction.action is 0
        
        Args:
            callback (Function): Register callback function, it takes transaction as argument
        """
        self.callback = callback
        
    def Generate_Transaction(self, order_id, amount, return_url=None):
        """_summary_

        Args:
            order_id (int): ID of order
            amount (int): amount of order to pay
            return_url (string, optional): Return url after payment. Defaults to None.

        Returns:
            string: Url for payment
        """        
        merchant_id = self.click_merchant_id
        service_id = self.click_service_id
        url = f"https://my.click.uz/services/pay?service_id={service_id}&merchant_id={merchant_id}&amount={amount}&transaction_param={order_id}"
        if return_url:
            url += f"&return_url={return_url}"
        return url
    


"""
Flask-ClickUz
-------------

This is the description for Flask-ClickUz
"""
from setuptools import setup

long_description = """
# Flask-ClickUz
> Only for Uzbekistan


Click.uz integration for Flask

## Links
* [About ClickUz](https://click.uz/)
* [ClickUz Docs](https://docs.click.uz/)

## How it Works

### Install

```
pip install Flask-ClickUz
```

### Add your credentials from ClickUz to config file

```python
CLICK_SECRET = 'Your secret key'
CLICK_SERVICE_ID = 'Your service ID'
CLICK_MERCHANT_ID = "Your merchant ID" 
```

### Create Flask App With Flask-ClickUz

```python
from flask import Flask
from flask_clickuz import Click
from flask_sqlalchemy import SQLAlchemy
db = SQLAlchemy()
click = Click(db)
orders = [525]

def Check_Allowment(data): 
    merchant_trans_id = data.get('merchant_trans_id')
    if int(merchant_trans_id) in orders:
        return True
    return False

def Click_Callback(transaction):
    action = transaction.action # 0 - cancaled, 1 - paid
    pass

def create_app():
    app = Flask(__name__)
    app.config.from_pyfile('config.py')
    db.init_app(app)
    
    click.Register_Callback(Click_Callback) # Register callback
    click.Register_Validator(Check_Allowment) # Register validator
    click.init_app(app)
    
    # Generate url for new orders
    url = click.Generate_Transaction(order_id=123, amount=50000, return_url = '/return_url')
    
    return app
```

### Flask-ClickUz models schema

```python

# Flask-ClickUz uses Flask-SQLAlchemy models to save data in database, it prefers to use Postgresql

class Click_Transaction(db.Model):
    __tablename__ = 'click_transaction'
    id = db.Column(db.Integer, primary_key=True)
    click_trans_id = db.Column(db.Integer, unique=True)
    service_id = db.Column(db.Integer)
    click_paydoc_id = db.Column(db.Integer)
    merchant_trans_id = db.Column(db.String(255))
    amount = db.Column(db.Float)
    action = db.Column(db.Integer)
    error = db.Column(db.Integer)
    error_note = db.Column(db.String(255))
    sign_time = db.Column(db.DateTime)
    sign_string = db.Column(db.String(255))
```

### Accessing Models
Select data from models

```python
click.model # Click_Transaction 
Click_Transaction = click.model
transaction = Click_Transaction.query.all() # You can select or filter data
```

Add model view to Flask-Admin

```python
from flask_admin.contrib.sqla import ModelView
admin.add_view(ModelView(click.model, db.session))
```



## Task List

### SHOP-API methods

- [x] Prepare
- [x] Complete


### Merchant API methods

- [ ] create_invoice
- [ ] check_invoice
- [ ] create_card_token
- [ ] verify_card_token
- [ ] payment_with_card_token
- [ ] delete_card_token
- [ ] check_payment
- [ ] merchant_trans_id
- [ ] cancel




## Licence
This project is licensed under the MIT License (see the `LICENSE` file for details).
"""

setup(
    name='Flask-ClickUz',
    version='1.0.1',
    url='https://github.com/Odya-LLC/flask_clickuz',
    license='MIT',
    author='odya',
    author_email='mmuhtor@gmail.com',
    description='Click.uz integration with Flask application, (only for Uzbekistan)',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=['flask_clickuz'],
    zip_safe=False,
    include_package_data=True,
    platforms=['3.6', '3.7', '3.8', '3.9', '3.10'],
    install_requires=[
        'Flask',"Flask-SQLAlchemy"
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)